import bpy
from bpy.app.handlers import persistent
import os
import shutil
from . import bl_info
from . utils.material import set_match_material_enum
from . utils.registration import get_path, reload_infotextures, reload_infofonts, reload_decal_libraries, reload_panels
from . utils.system import abspath


@persistent
def update_match_material_enum(none):
    set_match_material_enum()


@persistent
def update_userdecallibs_enum(none):
    scene = bpy.context.scene

    if scene.userdecallibs not in [lib[0] for lib in bpy.types.Scene.userdecallibs[1]['items']]:
        print("Note, the userdecallib defined in this blend file, could not be found among the registered decal libaries. Updating property by reloading decal libraries. Re-saving blend file is recommended.")
        reload_decal_libraries()


@persistent
def update_infodecal_sources(none):
    wm = bpy.context.window_manager

    # look for new images until found or until the last op is no longer machin3.load_images
    if wm.collectinfotextures:
        lastop = wm.operators[-1] if wm.operators else None
        newimages = ([img for img in bpy.data.images if img.name not in wm.excludeimages and len(img.name) < 63 and img.name.endswith(".png")])

        # stop looking
        if (lastop and lastop.bl_idname != 'MACHIN3_OT_load_images') or newimages:
            wm.collectinfotextures = False
            wm.excludeimages.clear()

            # copy new images to the infotextures location, remove the image data blocks and reload the infotexturees
            if newimages:
                createpath = os.path.join(get_path(), "assets", "Create")
                infopath = os.path.join(createpath, "infotextures")

                default = newimages[-1].name

                for img in newimages:
                    shutil.copy(abspath(img.filepath), os.path.join(infopath, img.name))
                    bpy.data.images.remove(img, do_unlink=True)

                reload_infotextures(default=default)

    # look for new fonts until found or until the last op is no longer machin3.load_fonts
    if wm.collectinfofonts:
        lastop = wm.operators[-1] if wm.operators else None
        newfonts = ([font for font in bpy.data.fonts if font.name not in wm.excludefonts and len(font.name) < 63])

        # stop looking
        if (lastop and lastop.bl_idname != 'MACHIN3_OT_load_fonts') or newfonts:
            wm.collectinfofonts = False
            wm.excludefonts.clear()

            # copy new fonts to the infofonts location, remove the font data blocks and reload the infofont
            if newfonts:
                createpath = os.path.join(get_path(), "assets", "Create")
                fontspath = os.path.join(createpath, "infofonts")

                default = newfonts[-1].name + ".ttf"

                for font in newfonts:
                    shutil.copy(abspath(font.filepath), os.path.join(fontspath, font.name + ".ttf"))
                    bpy.data.fonts.remove(font, do_unlink=True)

                reload_infofonts(default=default)


@persistent
def update_panels(none):
    reload_panels()

    scene = bpy.context.scene
    revision = bl_info.get("revision")

    if revision and not scene.DM.revision:
        scene.DM.revision = revision
