import bpy
from bpy.props import StringProperty
from bl_ui.space_toolsystem_common import ToolSelectPanelHelper
from ... utils.registration import get_prefs
from ... utils.ui import get_keymap_item


class CallDecalPie(bpy.types.Operator):
    bl_idname = "machin3.call_decal_pie"
    bl_label = "MACHIN3: Call DECALmachine Pie"
    bl_options = {'REGISTER', 'UNDO'}

    idname: StringProperty()

    def invoke(self, context, event):
        # reset decalmode, if it's NONE (the AddDecalToLibrary tool, if aborted, leaves it in this state)
        if get_prefs().decalmode == "NONE":
            get_prefs().decalmode = "INSERT"
            get_prefs().decalremovemode = False

        current_tool = ToolSelectPanelHelper._tool_get_active(context, 'VIEW_3D', None)[0][0]

        # check if both - DM and BC - use the D keymap
        dm_kmi = get_keymap_item('Object Mode', 'machin3.call_decal_pie', 'D')
        bc_kmi = get_keymap_item('3D View Tool: BoxCutter', 'wm.call_menu_pie', 'D', properties=[('name', 'BC_MT_pie')])

        # only if the current tool is BoxCutter and both keymaps are D, should you passthrough and let BC take over(and not display the decal pie)
        if current_tool == 'BoxCutter' and dm_kmi and bc_kmi:
            return {'PASS_THROUGH'}

        else:
            context.window_manager.decal_mousepos = (event.mouse_region_x, event.mouse_region_y)

            bpy.ops.wm.call_menu_pie(name='MACHIN3_MT_%s' % (self.idname))
            return {'FINISHED'}
